@@ -1,4 +1,4 @@
-//@line 38 "/home/kojima/Thunderbird/build/mail/base/content/msgHdrViewOverlay.js"
+//@line 38 "/home/karma/src/build/build/mail/base/content/msgHdrViewOverlay.js"
 
 
 /* This is where functions related to displaying the headers for a selected message in the
@@ -1070,19 +1070,27 @@ function detachAttachment(aAttachment, aSaveFirst)
                            aAttachment.messageUri, aSaveFirst);
 }
 
+/**
+ * Return true if possible attachments in the currently loaded message can be
+ * deleted/detached.
+ */
 function CanDetachAttachments()
 {
   var uri = GetLoadedMessage();
   var canDetach = !IsNewsMessage(uri) && (!IsImapMessage(uri) || MailOfflineMgr.isOnline());
   if (canDetach && ("content-type" in currentHeaderData))
-  {
-    var contentType = currentHeaderData["content-type"].headerValue;
-    canDetach = contentType.indexOf("application/x-pkcs7-mime") < 0 && 
-        contentType.indexOf("application/x-pkcs7-signature") < 0;
-  }
+    canDetach = !ContentTypeIsSMIME(currentHeaderData["content-type"].headerValue);
   return canDetach;
 }
 
+/** Return true if the content type is an S/MIME one. */
+function ContentTypeIsSMIME(contentType)
+{
+// S/MIME is application/pkcs7-mime and application/pkcs7-signature
+// - also match application/x-pkcs7-mime and application/x-pkcs7-signature.
+  return /application\/(x-)?pkcs7-(mime|signature)/.test(contentType);
+}
+
 function onShowAttachmentContextMenu()
 {
   // if no attachments are selected, disable the Open and Save...
@@ -1090,49 +1098,58 @@ function onShowAttachmentContextMenu()
   var selectedAttachments = attachmentList.selectedItems;
   var openMenu = document.getElementById('context-openAttachment');
   var saveMenu = document.getElementById('context-saveAttachment');
+  var saveAllMenu = document.getElementById('context-saveAllAttachments');
   var detachMenu = document.getElementById('context-detachAttachment');
   var deleteMenu = document.getElementById('context-deleteAttachment');
   var detachAllMenu = document.getElementById('context-detachAllAttachments');
   var deleteAllMenu = document.getElementById('context-deleteAllAttachments');
+
   var canDetach = CanDetachAttachments();
-  var canOpen = false;
-  for (var i = 0; i < selectedAttachments.length && !canOpen; i++)
-    canOpen = selectedAttachments[i].attachment.contentType != 'text/x-moz-deleted';
-  if (canOpen && selectedAttachments.length == 1)
+  var deletedAmongSelected = false;
+  var detachedAmongSelected = false;
+  var anyDeleted = false; // at least one deleted attachment in the list
+  var anyDetached = false; // at least one detached attachment in the list
+  var selectNone = selectedAttachments.length == 0;
+
+  // Check if one or more of the selected attachments are deleted.
+  for (var i = 0; i < selectedAttachments.length && !deletedAmongSelected; i++)
+    deletedAmongSelected =
+      (selectedAttachments[i].attachment.contentType == 'text/x-moz-deleted');
+
+  // Check if one or more of the selected attachments are detached.
+  for (var i = 0; i < selectedAttachments.length && !detachedAmongSelected; i++)
+    detachedAmongSelected = selectedAttachments[i].attachment.isExternalAttachment;
+
+  // Check if any attachments are deleted.
+  for (var i = 0; i < currentAttachments.length && !anyDeleted; i++)
+    anyDeleted = (currentAttachments[i].contentType == 'text/x-moz-deleted');
+
+  // Check if any attachments are detached.
+  for (var i = 0; i < currentAttachments.length && !anyDetached; i++)
+    anyDetached = currentAttachments[i].isExternalAttachment;
+
+  openMenu.setAttribute('disabled', selectNone);
+  saveMenu.setAttribute('disabled', selectNone);
+  detachMenu.setAttribute('disabled', selectNone);
+  deleteMenu.setAttribute('disabled', selectNone);
+  saveAllMenu.setAttribute('disabled', !selectNone);
+  detachAllMenu.setAttribute('disabled', !selectNone);
+  deleteAllMenu.setAttribute('disabled', !selectNone);
+
+  if (!selectNone)
   {
-    openMenu.removeAttribute('disabled');
+    openMenu.setAttribute('disabled', deletedAmongSelected);
+    saveMenu.setAttribute('disabled', deletedAmongSelected);
+    detachMenu.setAttribute('disabled', !canDetach || deletedAmongSelected
+                                        || detachedAmongSelected);
+    deleteMenu.setAttribute('disabled', !canDetach || deletedAmongSelected
+                                        || detachedAmongSelected);
   }
   else
   {
-    openMenu.setAttribute('disabled', true);
-  }
-  if (canOpen)
-  {
-    saveMenu.removeAttribute('disabled');
-  }
-  else
-  {
-    saveMenu.setAttribute('disabled', true);
-  }
-  if (canDetach && canOpen)
-  {
-    detachMenu.removeAttribute('disabled');
-    deleteMenu.removeAttribute('disabled');
-  }
-  else
-  {
-    detachMenu.setAttribute('disabled', 'true');
-    deleteMenu.setAttribute('disabled', 'true');
-  }
-  if (canDetach)
-  {
-    detachAllMenu.removeAttribute('disabled');
-    deleteAllMenu.removeAttribute('disabled');
-  }
-  else
-  {
-    detachAllMenu.setAttribute('disabled', 'true');
-    deleteAllMenu.setAttribute('disabled', 'true');
+    saveAllMenu.setAttribute('disabled', anyDeleted);
+    detachAllMenu.setAttribute('disabled', !canDetach || anyDeleted || anyDetached);
+    deleteAllMenu.setAttribute('disabled', !canDetach || anyDeleted || anyDetached);
   }
 }
 
@@ -1297,26 +1314,25 @@ function FillAttachmentListPopup(popup)
   // First clear out the old view...
   ClearAttachmentMenu(popup);
 
+  var canDetachOrDeleteAll = CanDetachAttachments();
+
   for (index in currentAttachments)
   {
     ++attachmentIndex;
     addAttachmentToPopup(popup, currentAttachments[index], attachmentIndex);
+    if (canDetachOrDeleteAll &&
+        (currentAttachments[index].isExternalAttachment ||
+        currentAttachments[index].contentType == 'text/x-moz-deleted'))
+      canDetachOrDeleteAll = false;
   }
 
   gBuildAttachmentPopupForCurrentMsg = false;
+
   var detachAllMenu = document.getElementById('file-detachAllAttachments');
   var deleteAllMenu = document.getElementById('file-deleteAllAttachments');
-  if (CanDetachAttachments())
-  {
-    detachAllMenu.removeAttribute('disabled');
-    deleteAllMenu.removeAttribute('disabled');
-  }
-  else
-  {
-    detachAllMenu.setAttribute('disabled', 'true');
-    deleteAllMenu.setAttribute('disabled', 'true');
-  }
 
+  detachAllMenu.setAttribute('disabled', !canDetachOrDeleteAll);
+  deleteAllMenu.setAttribute('disabled', !canDetachOrDeleteAll);
 }
 
 // Public method used to clear the file attachment menu
@@ -1394,20 +1410,7 @@ function addAttachmentToPopup(popup, attachment, attachmentIndex)
       if (!gDeleteLabelAccesskey)
         gDeleteLabelAccesskey = gMessengerBundle.getString("deleteLabelAccesskey");
 
-      // we should also check if an attachment has been detached...
-      // but that uses X-Mozilla-External-Attachment-URL, which
-      // we'd need to check for somehow.
-
-      var signedOrEncrypted = false;
-      if ("content-type" in currentHeaderData)
-      {
-        var contentType = currentHeaderData["content-type"].headerValue;
-        signedOrEncrypted = contentType.indexOf("application/x-pkcs7-mime") >= 0 || 
-            contentType.indexOf("application/x-pkcs7-signature") >= 0;
-      }
-      var canDetach = !(/news-message:/.test(attachment.uri)) && 
-          !signedOrEncrypted &&
-          (!(/imap-message/.test(attachment.uri)) || MailOfflineMgr.isOnline());
+      var canDetach = CanDetachAttachments() && !attachment.isExternalAttachment;
       menuitementry.setAttribute('label', gOpenLabel); 
       menuitementry.setAttribute('accesskey', gOpenLabelAccesskey); 
       menuitementry = openpopup.appendChild(menuitementry);
